# -*- coding: utf-8 -*-

"""Package for fao56_model"""

__author__ = "Stephan Smit"
__email__ = "stephansmit@hotmail.com"
# Do not edit this string manually, always use bumpversion
# Details in CONTRIBUTING.md
__version__ = "0.1.1"


def get_module_version():
    return __version__
